  
/* readBoost.cpp
 * A Csound API example with BOOST.
 * Creates files with random numbers using
 * BOOST random generator. Then runs a 
 * Csound tread calling 
 * readBoost.csd to play those files.
 */

//g++ -Wall -w -O2 -DUSE_DOUBLE -I/opt/csound6-git -I/opt/csound6-git/include  -o readBoost  readBoost.cpp -L. -lcsound64 -lsndfile -ldl -lm -lpthread -lfftw3

#include <stdio.h>
#include "csound.hpp"
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <ctime>   // std::time

//see /usr/include/boost/random
//this is Boost vers. 1.44
#include <boost/nondet_random.hpp>
#include <boost/random.hpp>
#include <boost/random/additive_combine.hpp>
#include <boost/random/bernoulli_distribution.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/cauchy_distribution.hpp>
#include <boost/random/discard_block.hpp>
#include <boost/random/exponential_distribution.hpp>
#include <boost/random/gamma_distribution.hpp>
#include <boost/random/geometric_distribution.hpp>
#include <boost/random/inversive_congruential.hpp>
#include <boost/random/lagged_fibonacci.hpp>
#include <boost/random/linear_congruential.hpp>
#include <boost/random/linear_feedback_shift.hpp>
#include <boost/random/lognormal_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>
#include <boost/random/poisson_distribution.hpp>
#include <boost/random/random_number_generator.hpp>
#include <boost/random/ranlux.hpp>
#include <boost/random/shuffle_output.hpp>
#include <boost/random/subtract_with_carry.hpp>
#include <boost/random/triangle_distribution.hpp>
#include <boost/random/uniform_01.hpp>
#include <boost/random/uniform_int.hpp>
#include <boost/random/uniform_on_sphere.hpp>
#include <boost/random/uniform_real.hpp>
#include <boost/random/uniform_smallint.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/random/xor_combine.hpp>

#include <boost/math/distributions.hpp> //additional distributions?

using namespace std;

uintptr_t csThread(void *clientData); 

typedef struct { 
  int result; 
  CSOUND *csound; 
  int PERF_STATUS; 
  //int initialized;
} userData; 

int main()
{
  int finish;
  void *ThreadID; 
  userData *ud; 
  ud = (userData *)malloc(sizeof(userData));  
  MYFLT *pvalue; 
  
  //adjust path to .csd as needed
   const char *argv[] = {"csound", "-+rtaudio=ALSA", "-odevaudio", "-s", "-d", "-b1024", "-B16384","/opt/csound6-git/MetaExs/readf/readBoost.csd" };

//-----------BOOST start here
	//set the path here where BOOST will write the files
	//adjust as needed
string path ("/opt/csound6-git/MetaExs/readf");

int k;
double x;	

  // Define a base random number generator and initialize it with a seed.
    // Always make sure seed argument unsigned for mt19937
    // Typical choices:  boost::mt19937, boost::ecuyer1988, boost::minstd_rand
    //boost::minstd_rand baseGen(1234u);
    boost::minstd_rand baseGen(static_cast<unsigned> (std::time(0)));

    //++++++++++++++++++++++++++++
    //++++++++++++++++++++++++++++
    // distribution implementations
    //++++++++++++++++++++++++++++
    //++++++++++++++++++++++++++++
    // Define distribution
        const double median = 1050.0;
    const double sigma = 600.0;
    boost::cauchy_distribution<> cauReal(median, sigma);

    //-----------------------
    // Define a random variate generator using our base generator and distribution
    boost::variate_generator<boost::minstd_rand&, boost::cauchy_distribution<> > cauRealGen(baseGen, cauReal);

    //------------------------
    // Get some random numbers:
    std::cout << endl;
    std::cout << "Ten samples of reals in a cauchy distribution:\n"<<endl;

//+++++++++++++++++++++++++++
//+++++++++++++++++++++++++++
//  file IO part
//+++++++++++++++++++++++++++
//+++++++++++++++++++++++++++

    string CauchyDatAppend ("/cauchy.");
    string CauchyDatPath = path + CauchyDatAppend;
    
    //adds numbers as file extension
    while (k < 5)
    {
    ofstream outf1(CauchyDatPath.c_str() + std::to_string(k)); 

    // ...stdout and also print to file
    for(int i = 0; i < 10; i++) {
        next:
        x = cauRealGen();
            if (x<=0 )
             goto next;
        //std::cout << cauRealGen() << '\n';
        std::cout << x << '\n';
        //print to file
        outf1 << x << '\n';   
    }
      std::cout << '\n';
      outf1.close();
      k++;
   } //end while

//---------BOOST, end here

  //csoundInitialize(ud->initialized);  
  ud->csound = csoundCreate(NULL);  
  //ud->result = csoundCompile(ud->csound, argc, argv); 
      ud->result=csoundCompile(ud->csound, 8, (char **)argv);

  if (!ud->result) {  
    ud->PERF_STATUS = 1; 
    ThreadID = csoundCreateThread(csThread, (void *)ud); 
  } 
  else { 
    return 1; 
  }  

  /* keep performing until user types a number and presses enter */
  scanf("%d", &finish);
  ud->PERF_STATUS = 0; 
  csoundDestroy(ud->csound); 
  free(ud);  
  return 0; 
} 

/* performance thread function */
uintptr_t csThread(void *data) 
{ 
  userData *udata = (userData *)data; 
  if (!udata->result) {
    while ((csoundPerformKsmps(udata->csound) == 0) &&
           (udata->PERF_STATUS == 1));
    csoundDestroy(udata->csound); 
  }        
  udata->PERF_STATUS = 0;    
  return 1; 
}  
